#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include "pvo.hh"
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"
#include "pvo-metadata.hh"

using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using namespace PVO::FITS;
using namespace PVO::Meta;
using std::string;



int main(int argc, char * argv[])
{
  /** 
   * PVOConeSearch is a function to do basic cone searching in PSRFITS files.
   *
   * This program is a command-line program that looks at all PSRFITS
   * files in the current directory (i.e. files that have extensions
   * .fits, .rf or .sf) and returns all those that cover sky within a
   * given search radius of a given RA,DEC position.
   *
   * The format of usage is: <br> 
   * <tt>> pvoconesearch RA DEC RADIUS</tt><br> 
   * RA and DEC can be in either decimal degrees or hms/dms
   * format (i.e.  hh:mm:ss.sss for RA and dd:mm:ss.sss for
   * DEC). Entering <br>
   * <tt>> pvoconesearch -h</tt><br>
   * will give a help message, using PVO::pvoconesearch_usage_text.
   *
   * The output is a list of valid files, their central RA & DEC,
   * their beam size, the central observing frequency, the DATE-OBS
   * keyword (time/date of observation), plus the angular offset [deg]
   * from the query position. The beam size of the FITS file is taken
   * into account, so that the central position can be more than
   * RADIUS away from the query position (as long as the query
   * position is still within the primary beam's FWHM).
   * 
   */


  bool verboseflag = false;

  std::vector<std::string> namelist;
  std::string RAstring,DECstring;
  double radius,RAcentre,DECcentre;
  unsigned int i,numColon;
  switch(argc){
  case 4:
    namelist = PVO::getAllFitsFiles();

    RAstring = argv[1];
    numColon=0;
    for(i=0;i<RAstring.size();i++) if(RAstring[i]==':') numColon++;
    if(numColon>0) RAcentre = dmsToDec(RAstring)*15.;
    else RAcentre = atof(argv[1]);
    DECstring = argv[2];
    numColon=0;
    for(i=0;i<DECstring.size();i++) if(DECstring[i]==':') numColon++;
    if(numColon>0) DECcentre = dmsToDec(DECstring);
    else DECcentre = atof(argv[2]);

    radius = atof(argv[3]);
    
    std::cout.setf(std::ios::fixed);
    std::cout << std::setprecision(4)
	      << "Searching with: RA = " << RAcentre
	      << " deg, DEC = " << DECcentre
	      << " deg, Radius = " << radius << " deg\n";

    break;
  default:
    if(argc==2 && strcmp(argv[1],"-h")==0)
      std::cout << pvoconesearch_usage_text;
    else
      std::cerr << pvoconesearch_usage_text_short;
    return 0;
    break;
  }

  ///////////////////////////////////////////
  // Now actually do the work...

  std::vector<Metadata> goodlist;

  for(i=0;i<namelist.size();i++){

    PSRFits fitsfile;
    fitsfile.setVerbosity(verboseflag);
    int status = fitsfile.openFile(namelist[i],READONLY);
  
    if(status==0){
      Metadata newmatch;
      newmatch.define(fitsfile);
      if(newmatch.close(RAcentre,DECcentre,radius)) goodlist.push_back(newmatch);
    }
  }

  std::stable_sort(goodlist.begin(),goodlist.end());

  printMetadataShort(std::cout,goodlist);

}
